PRO easegrid,iopt,alat,alon,thelon,thelat,ascale,bscale
;
;	COMPUTES THE FORWARD "EASE2" GRID TRANSFORM
;
;	GIVEN A LAT,LON (ALAT,ALON) AND THE SCALE (ASCALE) THE IMAGE
;	TRANSFORMATION COORDINATES (THELON,THELAT) ARE COMUTED
;	USING THE "EASE GRID" (VERSION 2.0) TRANSFORMATION GIVEN IN FORTRAN
;	SOURCE CODE SUPPLIED BY MJ BRODZIK AT NSIDC.
;	RADIUS EARTH=6378.137 KM (WGS 84)
;	MAP ECCENTRICITY=0.081819190843 (WGS84)
;
;	inputs:
;	  iopt: projection type 8=EASE2 N, 9-EASE2 S, 10=EASE2 T/M
;	  alon, alat: lon, lat (deg) to convert (can be outside of image)
;          ascale and bscale should be integer valued)
;	  ascale: grid scale factor (0..5)  pixel size is (bscale/2^ascale)
;	  bscale: base grid scale index (ind=int(bscale))
;
;         for definitions of isc and ind see ease2_map_info
;
;	outputs:
;	  thelon: X coordinate in pixels (can be outside of image)
;	  thelat: Y coordinate in pixels (can be outside of image)
;
        DTR=0.01745329241994

	ind=int(bscale)
	isc=int(ascale)
	dlon=alon
	phi= DTR * alat

        EASE2_MAP_INFO(iopt,isc,ind, $
         map_equatorial_radius_m, map_eccentricity, $
          e2, map_reference_latitude, map_reference_longitude, $
          map_second_reference_latitude, sin_phi1, cos_phi1, kz, $ 
          map_scale, bcols, brows, r0, s0, epsilon

	dlon = dlon - map_reference_longitude
	dlon = easeconv_normalize_degrees( dlon )
	lam = DTR * dlon
    
	sin_phi=sin(phi)
	q=(1.0d0-e2)*((sin_phi/(1.0d0-e2*sin_phi*sin_phi)) $
             - (1.0d0/(2.0d0*map_eccentricity)) $
             * log((1.0d0-map_eccentricity*sin_phi) $
             / ( 1.0d0+map_eccentricity*sin_phi)))

	IF IOPT EQ 8 THEN BEGIN		; EASE2 GRID NORTH
	   qp = 1.0d0-((1.0d0-e2)/(2.0d0*map_eccentricity) $
     		   * log((1.0d0-map_eccentricity) $
                  / (1.0d0 + map_eccentricity))) $
	   if (abs(qp-q) lt epsilon) then $
	      rho = 0.0
	   else $
	      rho = map_equatorial_radius_m * sqrt( qp - q )
	   x =  rho * sin( lam )
	   y = -rho * cos( lam )
	ENDIF ELSE IF IOPT EQ 9 THEN BEGIN	; EASE2 GRID SOUTH
	   qp = 1.0d0-((1.0d0-e2)/( 2.0d0*map_eccentricity) $
     		   * log((1.0d0 - map_eccentricity) $
     			  / (1.0d0 + map_eccentricity)))
	   if ( abs( qp + q ) lt epsilon ) then $
	     rho = 0.0
	   else $
	     rho = map_equatorial_radius_m * sqrt( qp + q )
	   x = rho * sin( lam )
	   y = rho * cos( lam )
	ENDIF ELSE IF IOPT EQ 10 THEN BEGIN	; EASE2 CYLINDRICAL
	   x =   map_equatorial_radius_m * kz * lam
	   y = ( map_equatorial_radius_m * q ) / ( 2.0D0 * kz )
        ENDIF

	thelon = r0 + ( x / map_scale ) + 0.5
	thelat = s0 + ( y / map_scale ) + 0.5 ; 0 at bottom

	END
